cat("Migrlib v1.2: R-functions for inferring patterns of migration \n")
cat("from gene frequncies under equilibrium conditions.\n")
# Set the R flag to TRUE if running under R, and to FALSE if running under S-plus 
if (is.null(version$language)) {R<-F
      cat("Running under S-plus. \n")} else 
{
   if (version$language=="R") 
      {R<-T
      cat("Running under R\n")}			
   else
      {stop("Sorry - I can't figure out if I'm running under R or S.  Contact theauthor!")}
}
if (R) dyn.load("./all.so") else dyn.load.shared("./all.o")
.C("initc")
	
absdet <- function(M) 
{
   prod(svd(M, nu=0, nv=0)$d)
}
absdet1 <- function(M) abs(prod(diag(qr(M)$qr)))

logdmultnorm <- function(x, C)                                 #R
{
   return(-0.5 * (t(x) %*% solve(C) %*% x) - 0.5*log(absdet1(C))) 
}

courgeau <- function(M,N)                                      #R
{
   n <- ncol(M)
   m <- n*(n+1)/2	
   A <- matrix(0.0,ncol=m,nrow=m)
   b <- matrix(0.0,ncol=1,nrow=m)
   z <- .C(NAME="rewrite1",M=M,N=N,A=A,b=b,n=n)
   C <- matrix(0.0,ncol=n,nrow=n)
   return(.C(NAME="backrewrite",C=C,x=solve(z$A,z$b),n=n)$C)
}


alternative2 <- function(M,N)
{
   Meigen <- eigen(M)
   MTeigen <- eigen(t(M))
   Q <- Meigen$vectors
   P <- MTeigen$vectors
   d <- Meigen$values
   Y <- solve(Q)%*%diag(1/(2*N))%*%P/(1-outer(d,d))
   return(Q%*%Y%*%solve(P))
}

forw2back <- function(M,N,incompatible=0)
{
   n <- length(N)
   Nmatrix <- matrix(rep(N,n),byrow=T,ncol=n)
   diag(Nmatrix)<-diag(Nmatrix)-incompatible
   M <- M*Nmatrix
   return(diag(1/apply(M,1,sum))%*%M)
}

logit <- function(p,lower=0) {log((p-lower)/(1-p))}
ilogit <- function(x,lower=0) {lower+(1-lower)/(1+exp(-x))}
log <- function(x,base=exp(1),lower=0) {
   if (missing(base))
	{ .Internal(log(x-lower)) }
   else 
	{ .Internal(log(x-lower, base)) }
}
ilog <- function(x,lower=0) {exp(x)+lower}

islandmodel <- function(theta,n,trans=F,start=F,convert=F)
{
   if (trans) {theta <- ilogit(theta,lower=.0001)}
   if (convert) {
    if (!trans) {theta < logit(theta,lower=.0001)}
    return(theta)
   }
   if (start) return(.1)	
   return((1-theta)*diag(n))
}

steppingstone <- function(theta,n,trans=F,start=F,convert=F)
{
   if (trans) {theta <- c(ilogit(theta[1],lower=.0001),ilogit(theta[2]))}
   if (convert) {
    if (!trans) {theta <- c(logit(theta[1],lower=.0001),logit(theta[2]))}
    return(theta)
   }   
   if (start) return(c(.1,.1))
   
   M <- matrix(0,ncol=n,nrow=n)
   diag(M) <- 1 - theta[2]
   M[abs(row(M)-col(M))==1] <- theta[2]/2
   M[1,1] <- M[n,n]  <- 1 - theta[2]/2
   M <- (1-theta[1])*M
   return(M)
}

Ns <- NA
Nh <- NA
sampled <- NA

checksetup <- function()
{
   n <- length(Ne)
   if (is.na(sampled[1])) {sampled <<- rep(T,n)} # Default is all subpops sampled
   nsampled <- sum(sampled)
   if (is.na(Nh[1])) {Nh <<- rep(Inf,nsampled)}      # and no sampling error
   if (is.na(Ns[1])) {Ns <<- rep(Inf,nsampled)}            #
}
	
simulatelocus <- function(M,Ne,q,pmean,w,tmin)
{
   # First simulate the process in the whole system
   n <- ncol(M)
   p <- rep(q,n)
   p <- .C(NAME="simproc",M=as.double(M),Ne=as.double(Ne),n=as.integer(n),p=as.double(p),pstop=as.double(pmean),tmin=as.integer(tmin))[[4]]

   # Then drop unsampled populations and add sampling variance
   p <- p[sampled]
   for (j in 1:length(p)){
      if (Nh[j]==Inf)
	 {if (Ns[j]!=Inf) 
	    {p[j] <- rbinom(1,p[j],size=2*Ns[j])/(2*Ns[j])}
	 # else no sampling error
	 }
      else # hypergeometric sampling error
	 {p[j] <- rhyper(nn=1,m=p[j]*2*Nh[j],n=(1-p[j])*2*Nh[j],k=2*Ns[j])/(2*Ns[j])} 
      NULL
      }
   return(p)
}
	
simulate <- function(M,Ne,nloci=1,q=.5,pmean=NA,w=NA,tmin=500)
{
   n <- length(Ne) # Number of subpopulations
   checksetup()
   nsampled <- sum(sampled) # Number of sampled populations

   if (is.na(w[1])) {w <- rep(1/nsampled,nsampled)} # Default is even weights

   # Unconditional or condtional simulations?
   if (is.na(pmean[1])) {pmean <- rep(0,nloci)}
   else {nloci <- length(pmean)}

   p <- matrix(q,ncol=nsampled,nrow=nloci)
   for (i in 1:nloci) {
      p[i,] <- simulatelocus(M,Ne,q,pmean[i],w,tmin)
      NULL
   }
   return(p)
}

weight <- function(C)                                     
{
   Cinv <- solve(C)
   apply(Cinv,1,sum)/sum(Cinv)
}

cov2corr <- function(C) C/sqrt(outer(diag(C),diag(C)))
	
logL <- function(theta)
{
   n <- length(Ne)                        
   M <- FFUN(theta,n,trans=T) # nlm passes the parameters transformed. Better tell this to FFUN
   C <- alternative2(M,Ne)[sampled,sampled] 
   n <- sum(sampled)  
   ssv <- (1/(2*Ns)-1/(2*Nh))/(1-1/(2*Nh))   
   diag(C) <- diag(C) + ssv
   Cinv <- solve(C)                                                    
   w <- apply(Cinv,1,sum)/sum(Cinv)
   pmean <- matrix(rep(pp%*%w,n),byrow=F,ncol=n)
   y <- as.matrix(((pp-pmean)/sqrt((pmean*(1-pmean))))[,-n]) 
   K <- diag(n)[-n,]-matrix(rep(w,n-1),byrow=T,nrow=n-1)     
   C <- K%*%C%*%t(K)                       
   l <- sum(apply(y,1,logdmultnorm,C))
   return(-l)			
}

if (!R) {nlm <- nlmin}

fitmodel <- function(FUN, p, start)
{
   if (missing(start)) {start <- FUN(start=T)}
   start <- FUN(start,trans=F,conv=T)
   checksetup()
   pp <<- p
   FFUN <<- FUN
   temp <- nlm(logL,start)
   temp$parameter <- FUN(temp$estimate,trans=T,conv=T)
   temp$objective <- -temp$minimum
   return(temp)
}

bootstrap <- function(FUN, theta, p, nboots=100, dots=F, file=F)
{
   checksetup()
   n <- length(Ne)                        
   M <- FUN(theta,n)
   C <- alternative2(M,Ne)[sampled,sampled] 
   ssv <- (1/(2*Ns)-1/(2*Nh))/(1-1/(2*Nh))   
   diag(C) <- diag(C) + ssv
   Cinv <- solve(C)                                                    
   wsub <- apply(Cinv,1,sum)/sum(Cinv)
   pmean <- p%*%wsub
   w <- rep(0,n)       
   w[sampled] <- wsub
	
   boots <- matrix(0.0,ncol=length(theta),nrow=nboots)
   for (i in 1:nboots) {
      if (dots) cat(".")
      p <- simulate(M,Ne,pmean=pmean,w=w)
      if (file) write(p,file=paste("simp",i,".txt",sep=""))
      boots[i,] <- fitmodel(FUN,p,start=theta)$par
      NULL
   }
   return(boots)
}

lrtestsummarize <- function(temp)
{
   cat("Likelihood-ratio test summary\n\n")
   Lambda <- temp$H1$obj-temp$H0$obj
   cat("Observed value of the test statistic: ",Lambda,"\n")
   nboots <- length(temp$lr)
   cat("Number of bootstrap samples: ",nboots,"\n")	
   pvalue <- sum(temp$lr > Lambda)/nboots
   cat("Significance probability: ",pvalue,"\n")
}
	
      
lrtest <- function(H0, H1, p, theta, nboots=100, dots=F, date=F, memory=F, file=F)
{
   checksetup()
   n <- length(Ne)

   fitH0 <- fitmodel(H0,p)
   fitH1 <- fitmodel(H1,p)
 	                        
   if (missing(theta)) {theta <- fitH0$par}

   M <- H0(theta,n)
   C <- alternative2(M,Ne)[sampled,sampled] 
   ssv <- (1/(2*Ns)-1/(2*Nh))/(1-1/(2*Nh))   
   diag(C) <- diag(C) + ssv
   Cinv <- solve(C)                                                    
   wsub <- apply(Cinv,1,sum)/sum(Cinv)
   pmean <- p%*%wsub
   w <- rep(0,n)       
   w[sampled] <- wsub
      
   lr <- rep(0.0,nboots)

   s0 <- theta;      
   s1 <- H1(start=T);

   for (i in 1:nboots) {
      if (dots) cat(".")
      if (memory) gc()
      p <- simulate(M,Ne,pmean=pmean,w=w)
      if (file) write(p,file=paste("simp",i,".txt",sep=""))
      lrH1 <- fitmodel(H1,p,start=s1)$objective 
      lrH0 <- fitmodel(H0,p,start=s0)$objective
      lr[i] <- lrH1-lrH0
      NULL
   }
   temp <- list(H0=fitH0,H1=fitH1,lr=lr)
   lrtestsummarize(temp)
   return(temp)
}

outer2 <- function(x) outer(x,x)

covobs <- function(p,w=rep(1/n,n))	
{
   n <- ncol(p)

   K <- diag(n)-matrix(rep(w,n),byrow=T,nrow=n) # Contrast transformation matrix     
   # observed covariances
   pmean <- matrix(rep(p%*%w,n),byrow=F,ncol=n)
   y <- (p-pmean)/sqrt(pmean*(1-pmean)) 
   z <- t(apply(y,1,outer2))
   dim(z) <- c(nrow(p),n,n)
   return(apply(z,c(2,3),mean))                    
}

covpred <- function(FUN,theta) 
{
   checksetup()
   n <- length(Ne)                        
   M <- FUN(theta,n)
   C <- alternative2(M,Ne)[sampled,sampled] 
   n <- sum(sampled)  
   ssv <- (1/(2*Ns)-1/(2*Nh))/(1-1/(2*Nh))   
   diag(C) <- diag(C) + ssv
   Cinv <- solve(C)                                                    
   w <- apply(Cinv,1,sum)/sum(Cinv)
   K <- diag(n)-matrix(rep(w,n),byrow=T,nrow=n)     
   return(K%*%C%*%t(K))                      
}

covplot <- function(Cpred,Cobs,file,mfcol=NA,ylim=c(-.13,.13))
{
   if (!missing(file)) { postscript(file=file,onefile=F,print.it=F,hori=F,width=7,height=6,paper="a4") }

   n<-nrow(Cpred)
   if (is.na(mfcol)) {
      nr <- trunc(sqrt(n)*1.8)
      nc <- trunc(sqrt(n)/1.8)
      if (nc*nr < n) {nc <- nc+1}
      if (nc*nr < n) {nr <- nr+1}
      mfcol<-c(nr,nc)
   }

   par(mfcol=mfcol,mar=c(1.2,1.2,.2,.2)+.1,cex=.5,mgp=c(1.2,.2,0),lab=c(21,3,7))

   for (i in 1:n){
      plot(1:n,Cpred[i,],xlab="",ylab=""
      ,type="l",lty=1,ylim=ylim)
      text(2,.1,as.character(i))
      lines(1:n,Cobs[i,],lty=2)
   }
   if (!missing(file)) {dev.off()}
}

# This returns the transition matrix in the recursion equation
# for the change in all covariances from on generation to
# the next
transition <- function(M,N)
{
   n <- ncol(M)
   m <- n*(n+1)/2	
   A <- matrix(0.0,ncol=m,nrow=m)
   b <- matrix(0.0,ncol=1,nrow=m)
   z <- .C(NAME="transition1",M=M,N=N,A=A,b=b,n=n)
   return(z$A)
}

# effectpop computes the total effective population size of
# the metapopulation from the first dominant eigenvalue
# of the transition matrix.  Note that this only works when the
# total population is complete isolated from the outside world
effectpop <- function(M,N) {
  if (!prod(apply(M,1,sum))) warning("effectpop: row sums of migration matrix do not sum to 1")
  A <- transition(M,N)
  return(1/(2*(1-eigen(A)$values[1])))
}

