#include "randlib.h"
#include <math.h>
#include <stdio.h>
#include <stdlib.h>

/* Simple binomial random number generation
long ignbin(long N,double p)
{
   long i,x;
   x=0;
   for (i=1;i<=N;i++) {
      if (rand()<p*RAND_MAX) x++;
   }
   return x;
} 
*/

void initc()
{
   srand(1);
}

/* ---------------------------------------------------------
   Simulate the process for 500 generations and
   until pmean passes pstop.  If pstop=0 then
   only the process is stopped after 500 gererations.
   All gene frequencies are initialized to .5 and
   the equilibrium is set to q = .5.  (Note that the 
   choice of q shouldn't matter provided that pmean
   is approximately sufficient for q.)
   Also Note the "-1"'s when referring to elements 
   of the various vectors and matrices passed as arguments.  
   --------------------------------------------------------- */


void simproc(double *M,double *N,long *n,double *p,double *pstop,long *tmin)
{
  long i,j,t;
  double pmean,pmean0,q;
  double P[100];
  double u[100];
  long stop;

  /* printf("n= %d pstop= %10.5f tmin=%d \n'",*n,*pstop,*tmin); */
  for (i=1; i<=*n; i++){
    u[i]=1.0;
    for (j=1;j<=*n;j++){
      u[i]-=M[i-1+(j-1)**n];
      /* printf("%5f ",M[i-1+(j-1)**n]); */
    }
  }

  q=(*pstop==0) ? 0.5 : *pstop;
  stop=0;
  for (t=1; !stop; t++){
    for (i=1;i<=*n;i++){
      P[i]=u[i]*q;                   /* Mutation/Long-range migration */
      for (j=1;j<=*n;j++){    
         P[i]+=M[i-1+(j-1)**n]*p[j-1];                    /* Migration */  
      }
      P[i]=ignbin((long)(2*N[i-1]),(float)P[i])/(2*N[i-1]); /* Drift */
    }
    for (i=1;i<=*n;i++) p[i-1]=P[i];
   
    pmean0=pmean;
    pmean=0;        
    for (i=1;i<=*n;i++) pmean+=P[i];
    pmean/=*n;
 
    /* Criterias for stopping the process */

    if (t<=*tmin)                       /* 500 generation must have pass*/
       stop=0;
    else if ((pmean==1.0)||(pmean==0.0)) /*There must be polymorphism*/ 
       stop=0;
    else if (*pstop==0)  /*If no pstop is supplied then stop anywhere*/
       stop=1;
    else {                    /*or stop if pstop has just been passed*/
       stop=((pmean>*pstop) == (pmean0<*pstop));
    }
  }
}

/* ----------------------------------------------
   Rewrite (7) to a big linear system
   ---------------------------------------------- */
void rewrite1(double *M,double *N,double *A,double *b,long *n)
{
  long i,j,l,k,r,s, m;
  double delta12N;
  m=*n*(*n+1)/2;
  r=0;
  for (i=1;i<=*n;i++) for (j=1;j<=i;j++){
     r++;
     if (i==j) 
       delta12N=1/(2 * N[i-1]);
     else
       delta12N=0;
     s=0;
     for (k=1;k<=*n;k++) for (l=1;l<=k;l++){
        s++;
        A[r-1+(s-1)*m] = M[i-1+(k-1)**n]* M[j-1+(l-1)**n] 
               + ((k!=l)?(M[i-1+(l-1)**n]* M[j-1+(k-1)**n]):0) 
	       + ((r==s)?(-1-delta12N):0);
     }
     b[r-1]= -delta12N;
  }
}
/* ----------------------------------------------
   Compute the transition matrix corresponding 
   to (7).  Note that this is slightly different 
   from the "A" when solving for the equilibrium
   solution....
   ---------------------------------------------- */
void transition1(double *M,double *N,double *A,double *b,long *n)
{
  long i,j,l,k,r,s, m;
  double delta12N;
  m=*n*(*n+1)/2;
  r=0;
  for (i=1;i<=*n;i++) for (j=1;j<=i;j++){
     r++;
     if (i==j) 
       delta12N=1/(2 * N[i-1]);
     else
       delta12N=0;
     s=0;
     for (k=1;k<=*n;k++) for (l=1;l<=k;l++){
        s++;
        A[r-1+(s-1)*m] = M[i-1+(k-1)**n]* M[j-1+(l-1)**n] 
               + ((k!=l)?(M[i-1+(l-1)**n]* M[j-1+(k-1)**n]):0) 
	  + ((r==s)?(/* -1 This is all the difference... */-delta12N):0);
     }
     b[r-1]= -delta12N;
  }
}
/* ----------------------------------------------
   Rewrite (18) to a big linear system
   ---------------------------------------------- */
void rewrite2(double *U, double *D,double *N,double *A,double *b,long *n)
{
  long i,j,l,k,r,s,m;
  m=*n*(*n+1)/2;
  r=0;
  for (i=1;i<=*n;i++) for (j=1;j<=i;j++){
     r++;
     b[r-1]=0;
     s=0;
     for (k=1;k<=*n;k++){
        for (l=1;l<=k;l++){
          s++;
          A[r-1+(s-1)*m] = U[i-1+(k-1)**n]* U[j-1+(l-1)**n] 
               + ((k!=l)?
                    (U[i-1+(l-1)**n]* U[j-1+(k-1)**n])
                   :(-D[i-1+(l-1)**n]* D[j-1+(k-1)**n]/(2*N[k-1]))) 
	       + ((r==s)?(-1):0);
        }
        b[r-1]-=D[i-1+(k-1)**n]*D[j-1+(k-1)**n]/(2*N[k-1]);
     }
  }
}

void backrewrite(double *C,double *x,long *n)
{
  long i,j,r;
  r=0;
  for (i=1;i<=*n;i++) for (j=1;j<=i;j++){
    r++;
    C[i-1+(j-1)**n]=C[j-1+(i-1)**n]=x[r-1];
  }
}

void testbin(long *N, double *p, long *X)
{
   
   *X= ignbin(*N,*p); 
}

static double dabs(double x)
{
  return (x>0?x:-x);
}

void ifd(double *u, double *N, long *n, double *alpha, double *beta, long *maxiter, double *deltamax)
{
   long i,j,iter;
   double sum,previous,palpha[50],delta;
   iter=0;
   do {
     *deltamax=0;

     /* Choose alpha's to satisfy ifd */
     for (i=1;i<=*n;i++){
        sum=0;
        for (j=1;j<=*n;j++){
           sum+=N[j-1]*u[i-1+(j-1)**n]*beta[j-1];
        }
        palpha[i]=alpha[i-1];
        alpha[i-1]=N[i-1]/sum;
     }
     /* Normalise the alpha's to sum to one (by convention) */
     sum=0;
     for (i=1;i<=*n;i++){
        sum+=alpha[i-1];
     }
     for (i=1;i<=*n;i++){
        alpha[i-1]/=sum;
	/*  printf("%15.10f",alpha[i-1]);*/

     /* Keep track of largest relative change */
        delta=dabs((alpha[i-1]-palpha[i])/palpha[i]);
        *deltamax=(delta>*deltamax)?delta:*deltamax;
     }

     /* Solve the beta part of the system  */
     for (j=1;j<=*n;j++){
        sum=0;
        for (i=1;i<=*n;i++){
           sum+=u[i-1+(j-1)**n]*alpha[i-1];
        }
        previous=beta[j-1];
        beta[j-1]=1/sum;
        /*printf("%15.10f",beta[j-1]);*/
        delta=dabs((beta[j-1]-previous)/previous);
        *deltamax=(delta>*deltamax)?delta:*deltamax;
     }
     /*printf("%15.10f \n",*deltamax);*/
     iter++;
   } while ((*deltamax>0.00000001) && (iter<*maxiter));
   /* printf("%d %15.12f \n",iter,*deltamax); */
}
